import time

from qgis.PyQt.QtCore import pyqtSignal
from qgis.PyQt.QtWidgets import QAction
from qgis.core import QgsApplication, QgsTask, Qgis, QgsMessageLog


def classFactory(iface):
    return TestPlugin(iface)

class TestTask(QgsTask):

    importComplete = pyqtSignal()
    errorOccurred = pyqtSignal()

    def __init__(self, name, delay):
        QgsTask.__init__(self, name)

        self.name = name
        self.delay = delay / 100.0

    def run(self):
        for i in range(100):
            if self.isCanceled():
                break
            time.sleep(self.delay)
            self.setProgress(i)

        return True

    def finished(self, result):
        QgsMessageLog.logMessage('In finished(), emit signal')
        if result:
            self.importComplete.emit()
        else:
            self.errorOccurred.emit()

class TestPlugin:
    def __init__(self, iface):
        self.iface = iface

    def initGui(self):
         self.action = QAction('Test', self.iface.mainWindow())
         self.iface.addPluginToMenu('Test', self.action)
         self.action.triggered.connect(self.run)

    def unload(self):
        self.iface.removePluginMenu('Test', self.action)

    def run(self):
        task = TestTask('my task', 10)
        #task.importComplete.connect(self.complete)
        #task.errorOccurred.connect(self.error)
        QgsApplication.taskManager().addTask(task)
        #QgsMessageLog.logMessage('Tasks: {}'.format(QgsApplication.taskManager().count()))

    def complete(self):
        QgsMessageLog.logMessage('Task finished')

    def error(self):
        QgsMessageLog.logMessage('Error occurred')
